@@ -2,11 +2,10 @@ package Pod::L10N::Html;
 use strict;
 require Exporter;
 
-use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
-$VERSION = '1.05';
-@ISA = qw(Exporter);
-@EXPORT = qw(pod2htmll10n htmlify);
-@EXPORT_OK = qw(anchorify);
+our $VERSION = '1.06';
+our @ISA = qw(Exporter);
+our @EXPORT = qw(pod2htmll10n htmlify);
+our @EXPORT_OK = qw(anchorify relativize_url);
 
 use Carp;
 use Config;
@@ -16,6 +15,7 @@ use File::Spec;
 use File::Spec::Unix;
 use Getopt::Long;
 use Pod::Simple::Search;
+use Pod::Simple::SimpleTree ();
 
 use Pod::L10N::Model;
 
@@ -99,6 +99,19 @@ This program is distributed under the Artistic License.
 
 =cut
 
+# This sub duplicates the guts of Pod::Simple::FromTree.  We could have
+# used that module, except that it would have been a non-core dependency.
+sub feed_tree_to_parser {
+    my($parser, $tree) = @_;
+    if(ref($tree) eq "") {
+	$parser->_handle_text($tree);
+    } elsif(!($tree->[0] eq "X" && $parser->nix_X_codes)) {
+	$parser->_handle_element_start($tree->[0], $tree->[1]);
+	feed_tree_to_parser($parser, $_) foreach @{$tree}[2..$#$tree];
+	$parser->_handle_element_end($tree->[0]);
+    }
+}
+
 # "
 
 my $Cachedir; 
@@ -152,7 +165,7 @@ sub init_globals {
     $Doindex = 1;               # non-zero if we should generate an index
     $Backlink = 0;              # no backlinks added by default
     $Header = 0;                # produce block header/footer
-    $Title = '';                # title to give the pod(s)
+    $Title = undef;             # title to give the pod(s)
 }
 
 sub pod2htmll10n {
@@ -218,44 +231,6 @@ sub pod2htmll10n {
         close $cache or die "error closing $Dircache: $!";
     }
 
-    # set options for the parser
-    my $parser = Pod::L10N::Html::LocalPodLinks->new();
-    $parser->codes_in_verbatim(0);
-    $parser->anchor_items(1); # the old Pod::Html always did
-    $parser->backlink($Backlink); # linkify =head1 directives
-    $parser->htmldir($Htmldir);
-    $parser->htmlfileurl($Htmlfileurl);
-    $parser->htmlroot($Htmlroot);
-    $parser->index($Doindex);
-    $parser->no_errata_section(!$Poderrors); # note the inverse
-    $parser->output_string(\my $output); # written to file later
-    $parser->pages(\%Pages);
-    $parser->quiet($Quiet);
-    $parser->verbose($Verbose);
-
-#    $parser->html_charset('UTF-8');
-    $parser->html_encode_chars('&<>">');
-#    $parser->html_header_tags('');
-
-    # XXX: implement default title generator in pod::simple::xhtml
-    # copy the way the old Pod::Html did it
-    $Title = html_escape($Title);
-
-    # We need to add this ourselves because we use our own header, not
-    # ::XHTML's header. We need to set $parser->backlink to linkify
-    # the =head1 directives
-    my $bodyid = $Backlink ? ' id="_podtop_"' : '';
-
-    my $csslink = '';
-    my $tdstyle = ' style="background-color: #cccccc; color: #000"';
-
-    if ($Css) {
-        $csslink = qq(\n<link rel="stylesheet" href="$Css" type="text/css" />);
-        $csslink =~ s,\\,/,g;
-        $csslink =~ s,(/.):,$1|,;
-        $tdstyle= '';
-    }
-
     my $input;
     unless (@ARGV && $ARGV[0]) {
         if ($Podfile and $Podfile ne '-') {
@@ -273,6 +248,69 @@ sub pod2htmll10n {
         $encoding = 'utf-8';
     }
 
+    # set options for input parser
+    my $parser = Pod::Simple::SimpleTree->new;
+    $parser->codes_in_verbatim(0);
+    $parser->accept_targets(qw(html HTML));
+    $parser->no_errata_section(!$Poderrors); # note the inverse
+
+    warn "Converting input file $Podfile\n" if $Verbose;
+    my $podtree = $parser->parse_string_document($content)->root;
+
+    unless(defined $Title) {
+	if($podtree->[0] eq "Document" && ref($podtree->[2]) eq "ARRAY" &&
+		$podtree->[2]->[0] eq "head1" && @{$podtree->[2]} == 3 &&
+		ref($podtree->[2]->[2]) eq "" && $podtree->[2]->[2] eq "NAME" &&
+		ref($podtree->[3]) eq "ARRAY" && $podtree->[3]->[0] eq "Para" &&
+		@{$podtree->[3]} >= 3 &&
+		!(grep { ref($_) ne "" }
+		    @{$podtree->[3]}[2..$#{$podtree->[3]}]) &&
+		(@$podtree == 4 ||
+		    (ref($podtree->[4]) eq "ARRAY" &&
+			$podtree->[4]->[0] eq "head1"))) {
+	    $Title = join("", @{$podtree->[3]}[2..$#{$podtree->[3]}]);
+	}
+    }
+
+    $Title //= "";
+    $Title = html_escape($Title);
+
+    # set options for the HTML generator
+    $parser = Pod::L10N::Html::LocalPodLinks->new();
+    $parser->codes_in_verbatim(0);
+    $parser->anchor_items(1); # the old Pod::Html always did
+    $parser->backlink($Backlink); # linkify =head1 directives
+    $parser->force_title($Title);
+    $parser->htmldir($Htmldir);
+    $parser->htmlfileurl($Htmlfileurl);
+    $parser->htmlroot($Htmlroot);
+    $parser->index($Doindex);
+    # still need as parse twice
+    $parser->no_errata_section(!$Poderrors); # note the inverse
+    $parser->output_string(\my $output); # written to file later
+    $parser->pages(\%Pages);
+    $parser->quiet($Quiet);
+    $parser->verbose($Verbose);
+
+#    $parser->html_charset('UTF-8');
+    $parser->html_encode_chars('&<>">');
+#    $parser->html_header_tags('');
+
+    # We need to add this ourselves because we use our own header, not
+    # ::XHTML's header. We need to set $parser->backlink to linkify
+    # the =head1 directives
+    my $bodyid = $Backlink ? ' id="_podtop_"' : '';
+
+    my $csslink = '';
+    my $tdstyle = ' style="background-color: #cccccc; color: #000"';
+
+    if ($Css) {
+        $csslink = qq(\n<link rel="stylesheet" href="$Css" type="text/css" />);
+        $csslink =~ s,\\,/,g;
+        $csslink =~ s,(/.):,$1|,;
+        $tdstyle= '';
+    }
+
     # header/footer block
     my $block = $Header ? <<END_OF_BLOCK : '';
 <table border="0" width="100%" cellspacing="0" cellpadding="3">
@@ -304,7 +342,6 @@ $block
 </html>
 HTMLFOOT
 
-    warn "Converting input file $Podfile\n" if $Verbose;
 #    $parser->parse_file($input);
     $parser->parse_string_document($content);
 #    $parser->html_charset($parser->encoding());
